/* $Id: gx.c,v 1.6 1998/11/05 02:06:23 marcus Exp $
******************************************************************************

   KGI - Cyrix MediaGX graphics (acceleration) driver

   Copyright (C) 1997-1998		Emmanuel Marty
   Copyright (C) 1997-1998		Cyrix Corporation

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

   This code was derived from the following sources of information :

   Cyrix MediaGX confidential databooks
   Cyrix MediaGX gxvideo.c example sourcecode
   Help and code from Sengan Baring-Gould of Cyrix Corporation

   Many thanks to the Cyrix Corporation for providing me with
   documentation and official support about the GX chipset family.

******************************************************************************
*/

#include <kgi/maintainers.h>
#define MAINTAINER	Emmanuel_Marty
#define DRIVER_NAME	"Cyrix MediaGX graphics driver"
#define DRIVER_REV	"$Revision: 1.6 $"

#include <kgi/module.h>
#include <Cyrix/gx.h>
#include "suggest.inc"

/*	Acceleration functions		*/

static int GX_drawbox (struct kgi_graphic *gr, void *iobuf) {
	IS_CRITICAL;
	struct kgi_box *boxp = (struct kgi_box *)iobuf;
	kgiu16 gt = gr->dev.mode.graphtype, color;
	mem_vaddr gxbase = ((struct chipset_private *)
		KGIM_PRIVATE (gr->dev.dpy, chipset))->gx_control.baseptr;
	kgiu32 lockval;

	if ((gt == KGIGT_4BIT) || (gt == KGIGT_TEXT16))
		return -E(ACCEL,NOSUP_MODE_MMAP);

	if (boxp->w && boxp->h) {
		color = gr->gc->fg_color;

		if (gt == KGIGT_8BIT)
			color |= (color << 8);

		while (READ_GX32 (gxbase, GP_BLIT_STATUS) & BS_BLIT_PENDING);

		ENTER_CRITICAL;
		lockval = READ_GX32 (gxbase, DC_UNLOCK);
		WRITE_GX32 (gxbase, DC_UNLOCK, DC_UNLOCK_VALUE);

		WRITE_GX32 (gxbase, GP_DST_XCOOR, boxp->x | (boxp->y << 16));
		WRITE_GX32 (gxbase, GP_WIDTH, boxp->w | (boxp->h << 16));
		WRITE_GX16 (gxbase, GP_PAT_COLOR_0, color);
		WRITE_GX16 (gxbase, GP_RASTER_MODE, 0xF0);
		WRITE_GX16 (gxbase, GP_BLIT_MODE, BM_WRITE_FB);

		WRITE_GX32 (gxbase, DC_UNLOCK, lockval);
		LEAVE_CRITICAL;

		/*	necessary before an LFB access, until GGI can fix it */
		while (READ_GX32 (gxbase, GP_BLIT_STATUS) & BS_BLIT_BUSY);
	}

	return 0;
}

/*	KGI module interface	*/

int kgim_accel_init (struct kgi_card *card, struct kgi_display *dpy) {
	NOTICE (DRIVER_NAME " rev " DRIVER_REV);

	return EOK;
}

void kgim_accel_done (struct kgi_display *dpy) {
	NOTICE (DRIVER_NAME " rev " DRIVER_REV " removed");
}

int kgim_accel_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
			      enum kgi_tm_cmd *cmd) {
	return EOK;
}

void kgim_accel_set_mode (struct kgi_display *dpy, struct kgi_mode *mode) {
}

int kgim_accel_command (struct kgi_graphic *gr, kgi_uint cmd, kgi_uint arg) {
	kgi_suggest *suggest;
	const char *name;

	if (!(gr->dev.state & KGI_MAPPED_ON_DISPLAY)) {
		/*	Don't accelerate unmapped screen	*/
		return -E(ACCEL,NOSUP_MODE_MMAP);
	}

	switch (cmd) {

		case ACCEL_GETSUGGEST:
			suggest = (kgi_suggest *) (gr->io_buf);
			gr->io_res = gr->io_buf;

			if ((suggest->id < 0) || (suggest->id > 4)) {
				return -E(ACCEL,BOGUS);
			}

			if (!(name = sugg_default[gr->dev.mode.graphtype].
			       text[suggest->id])) {

				return -E(ACCEL,BOGUS);
			}

			strcpy (suggest->name, name);
			suggest->args[0] = 0;
			suggest->id++;

			if ( (suggest->id >= 4) ||
			     (sugg_default[gr->dev.mode.graphtype].
			      text[suggest->id] == NULL) )

				suggest->id = 0;

			return 0;

		case ACCEL_DRAWBOX:
			return (GX_drawbox (gr, gr->io_buf));

		case ACCEL_FILLSCREEN:
			return -E(ACCEL,NOSUP_ALWAYS_LOWER);

		default:
			return -E(ACCEL,NOSUP_ALWAYS_MMAP);

	}
}
